# Copyright (c) 2008-2009, samurai-x.org
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the samurai-x.org nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY SAMURAI-X.ORG ``AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL SAMURAI-X.ORG  BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from .memstream import MemoryInputStream

class Protobj(object):
    """
        Base class for all protocol objects.

        .. data:: create_lazy

            None as default, but it can be a classmethod used
            to create a Protobj instance lazily for convenience.

            :see: :func:`ooxcb.builder.build_list`

        .. data:: pythonize_lazy

            None as default, but if it can be a method used to
            pythonize a protocol object lazily if it's inside
            a list.

            :see: :class:`ooxcb.list.List`

    """
    create_lazy = None # TODO: does it have to be defined here?
    pythonize_lazy = None

    def __init__(self, conn):
        self.conn = conn

    def read(self, stream):
        """
            Placeholder for subclasses.
            :param stream: an object providing a buffer interface. Read
                           myself from that stream!
        """
        raise NotImplementedError()

    def read_from_address(self, address):
        """
            parse the memory at *address*
        """
        return self.read(MemoryInputStream(address))

    @classmethod
    def create_from_address(cls, conn, address):
        """
            :type conn: :class:`ooxcb.conn.Connection`
            :param address: an int pointing to the data in the
                            memory
        """
        self = cls(conn)
        self.read_from_address(address)
        return self

    @classmethod
    def create_from_stream(cls, conn, stream):
        """
            :type conn: :class:`ooxcb.conn.Connection`
            :param stream: a stream-like object to read from.
        """
        self = cls(conn)
        self.read(stream)
        return self

